import subprocess
import json


class fdtCLU:
    def run_fab_command(
        self,
        command,
        capture_output: bool = False,
        silently_continue: bool = False,
        raw_output: bool = False,
    ):
        result = subprocess.run(
            ["fab", "-c", command], capture_output=capture_output, text=True
        )
        if not (silently_continue) and (result.returncode > 0 or result.stderr):
            raise Exception(
                f"Error running fab command. exit_code: '{result.returncode}'; stderr: '{result}'"
            )
        if capture_output and not raw_output:
            output = result.stdout.strip()
            return output
        elif capture_output and raw_output:
            return result

    def fab_get_workspace_id(self, name):
        result = self.run_fab_command(
            f"get /{name} -q id", capture_output=True, silently_continue=True
        )
        return result

    def fab_workspace_exists(self, name):
        id = self.run_fab_command(
            f"get /{name} -q id", capture_output=True, silently_continue=True
        )
        return id

    def fab_get_id(self, name):
        id = self.run_fab_command(
            f"get /{self.trg_workspace_name}/{name} -q id",
            capture_output=True,
            silently_continue=True,
        )
        return id

    def fab_get_item(self, name):
        item = self.run_fab_command(
            f"get /{self.trg_workspace_name}/{name}",
            capture_output=True,
            silently_continue=True,
        )
        return item

    def fab_get_eventstream_connection_string(self, name, connection_name):
        connection_id = ""
        item_id = self.fab_get_id(name)

        item = self.run_fab_command(
            f"api -X get /workspaces/{self.trg_workspace_id}/eventstreams/{item_id}/topology",
            capture_output=True,
            silently_continue=True,
        )
        topology = json.loads(item)

        sources = topology.get("text", {}).get("sources", [])
        source_id = list(
            filter(lambda source: source["name"] == connection_name, sources)
        )
        if len(source_id):
            connection_id = source_id[0].get("id")

        destinations = topology.get("text", {}).get("destinations", [])
        destination_id = list(
            filter(
                lambda destination: destination["name"] == connection_name, destinations
            )
        )
        if len(destination_id):
            connection_id = destination_id[0].get("id")

        connection = self.run_fab_command(
            f"api -X get /workspaces/{self.trg_workspace_id}/eventstreams/{item_id}/sources/{connection_id}/connection",
            capture_output=True,
            silently_continue=True,
        )
        connection = json.loads(connection)
        connection = (
            connection.get("text", {})
            .get("accessKeys", {})
            .get("primaryConnectionString")
        )
        return connection

    def fab_get_display_name(self, name):
        display_name = self.run_fab_command(
            f"get /{self.trg_workspace_name}/{name} -q displayName",
            capture_output=True,
            silently_continue=True,
        )
        return display_name

    def fab_get_kusto_query_uri(self, name):
        connection = self.run_fab_command(
            f"get /{self.trg_workspace_name}/{name} -q properties.queryServiceUri -f",
            capture_output=True,
            silently_continue=True,
        )
        return connection

    def fab_get_kusto_ingest_uri(self, name):
        connection = self.run_fab_command(
            f"get /{self.trg_workspace_name}/{name} -q properties.ingestionServiceUri -f",
            capture_output=True,
            silently_continue=True,
        )
        return connection

    def fab_create_folder(self, display_name):
        folder = {"displayName": display_name}

        workspace_id = self.fab_get_workspace_id(self.trg_workspace_name)
        folderResponse = self.run_fab_command(
            f"api -X post /workspaces/{workspace_id}/folders/ -i {json.dumps(folder)}",
            capture_output=True,
            silently_continue=True,
        )
        folder_id = self.fab_get_folder_id(display_name)

        return folder_id

    def fab_update_item_folder(self, item, display_name):

        workspace_id = self.fab_get_workspace_id(self.trg_workspace_name)
        item_id = self.fab_get_id(item)
        folder_id = self.fab_create_folder(display_name)

        folder = {"targetFolderId": folder_id}

        updateFolderResponse = self.run_fab_command(
            f"api -X post /workspaces/{workspace_id}/items/{item_id}/move -i {json.dumps(folder)}",
            capture_output=True,
            silently_continue=True,
        )

    def fab_get_folders(self):
        workspace_id = self.fab_get_workspace_id(self.trg_workspace_name)
        response = self.run_fab_command(
            f"api workspaces/{workspace_id}/folders",
            capture_output=True,
            silently_continue=True,
        )
        return json.loads(response).get("text", {}).get("value", [])

    def fab_get_folder_id(self, display_name):
        folders = self.fab_get_folders()

        return (
            list(filter(lambda x: x["displayName"] == display_name, folders))
            .pop()
            .get("id")
        )



import json


class fdtItemManagement:
    def replace_kqldb_parent_eventhouse(self, folder_path, parent_eventhouse):
        property_file = f"{folder_path}/DatabaseProperties.json"
        with open(property_file, "r", encoding="utf-8") as file:
            content = json.load(file)
            content["parentEventhouseItemId"] = self.fab_get_id(parent_eventhouse)
        with open(property_file, "w", encoding="utf-8") as file:
            json.dump(content, file, indent=4)

    def replace_eventstream_destination(self, folder_path, it_destinations):
        property_file = f"{folder_path}/eventstream.json"
        with open(property_file, "r", encoding="utf-8") as file:
            content = json.load(file)
            destinations = content.get("destinations", [])
            for destination in destinations:
                if destination.get("type") != "CustomEndpoint":
                    filtered_data = list(
                        filter(
                            lambda table: table["name"] == destination.get("name")
                            and table["type"] == destination.get("type"),
                            it_destinations,
                        )
                    )
                    if len(filtered_data) > 0:
                        destination["properties"]["workspaceId"] = (
                            self.get_mapping_table_new_from_type_item(
                                "Workspace Id", self.trg_workspace_name
                            )
                        )
                        destination["properties"]["itemId"] = (
                            self.get_mapping_table_new_from_type_item(
                                "KQL DB ID", filtered_data[0].get("itemName")
                            )
                        )
                        if (
                            destination.get("properties", {}).get("databaseName")
                            is not None
                        ):
                            destination["properties"]["databaseName"] = (
                                self.get_mapping_table_new_from_type_item(
                                    "KQL DB Name", filtered_data[0].get("itemName")
                                )
                            )
        with open(property_file, "w", encoding="utf-8") as file:
            json.dump(content, file, indent=4)

    def replace_kqldashboard_datasources(self, folder_path, it_datasources):
        property_file = f"{folder_path}/RealTimeDashboard.json"
        with open(property_file, "r", encoding="utf-8") as file:
            content = json.load(file)
            datasources = content.get("dataSources", [])
            for datasource in datasources:
                filtered_data = list(
                    filter(
                        lambda table: table["name"] == datasource.get("name"),
                        it_datasources,
                    )
                )
                if len(filtered_data) > 0:
                    datasource["workspace"] = self.get_mapping_table_new_from_type_item(
                        "Workspace Id", self.trg_workspace_name
                    )
                    datasource["database"] = self.get_mapping_table_new_from_type_item(
                        "KQL DB ID", filtered_data[0].get("itemName")
                    )
                    datasource["clusterUri"] = self.get_mapping_table_parent_type(
                        "KQL DB Eventhouse",
                        filtered_data[0].get("itemName"),
                        "Kusto Query Uri",
                    )
        with open(property_file, "w", encoding="utf-8") as file:
            json.dump(content, file, indent=4)

    def replace_kqlqueryset_datasources(self, folder_path, it_datasources):
        property_file = f"{folder_path}/RealTimeQueryset.json"
        with open(property_file, "r", encoding="utf-8") as file:
            content = json.load(file)
            datasources = content.get("queryset", {}).get("dataSources", [])
            for datasource in datasources:
                filtered_data = list(
                    filter(
                        lambda table: str(table["itemName"]).replace(".KQLDatabase", "")
                        == datasource.get("databaseItemName"),
                        it_datasources,
                    )
                )
                if len(filtered_data) > 0:
                    datasource["databaseItemId"] = (
                        self.get_mapping_table_new_from_type_item(
                            "KQL DB ID", filtered_data[0].get("itemName")
                        )
                    )
                    datasource["clusterUri"] = self.get_mapping_table_parent_type(
                        "KQL DB Eventhouse",
                        filtered_data[0].get("itemName"),
                        "Kusto Query Uri",
                    )
        with open(property_file, "w", encoding="utf-8") as file:
            json.dump(content, file, indent=4)

    def replace_pbi_report_definition(self, folder_path, datasource):
        property_file = f"{folder_path}/definition.pbir"
        sm_name = datasource.replace(".SemanticModel", "")
        ws_id = self.get_mapping_table_new_from_type("Workspace Id")
        sm_id = self.get_mapping_table_new_from_type_item(
            "Semantic Model ID", datasource
        )
        pbir_definition = {
            "$schema": "https://developer.microsoft.com/json-schemas/fabric/item/report/definitionProperties/1.0.0/schema.json",
            "version": "4.0",
            "datasetReference": {
                "byPath": None,
                "byConnection": {
                    "connectionString": f"Data Source=powerbi://api.powerbi.com/v1.0/myorg/{ws_id};Initial Catalog={sm_name};Integrated Security=ClaimsToken",
                    "pbiServiceModelId": None,
                    "pbiModelVirtualServerName": "sobe_wowvirtualserver",
                    "pbiModelDatabaseName": sm_id,
                    "connectionType": "pbiServiceXmlaStyleLive",
                    "name": "EntityDataSource",
                },
            },
        }
        with open(property_file, "w", encoding="utf-8") as file:
            json.dump(pbir_definition, file, indent=4)

    def replace_pipeline_parameter(self, folder_path, it_parameters):
        property_file = f"{folder_path}/pipeline-content.json"
        with open(property_file, "r", encoding="utf-8") as file:
            content = json.load(file)
            properties = content.get("properties", {}).get("parameters", {})
            for parameter in it_parameters:
                if parameter["type"] == "kusto_query_uri":
                    pipeline_parameter = properties.get(parameter["name"], {})
                    if ".KQLDatabase" in parameter["source"]:
                        pipeline_parameter["defaultValue"] = (
                            self.get_mapping_table_parent_type(
                                "KQL DB Eventhouse",
                                parameter["source"],
                                "Kusto Query Uri",
                            )
                        )
                    elif ".Eventhouse" in parameter["source"]:
                        pipeline_parameter["defaultValue"] = (
                            self.get_mapping_table_new_from_type_item(
                                "Kusto Query Uri",
                                (
                                    parameter["source"]
                                    if self.eventhouse_name == ""
                                    or self.eventhouse_name is None
                                    else f"{self.eventhouse_name}.Eventhouse"
                                ),
                            )
                        )
                elif parameter["type"] == "kusto_ingest_uri":
                    pipeline_parameter = properties.get(parameter["name"], {})
                    if ".KQLDatabase" in parameter["source"]:
                        pipeline_parameter["defaultValue"] = (
                            self.get_mapping_table_parent_type(
                                "KQL DB Eventhouse",
                                parameter["source"],
                                "Kusto Ingest Uri",
                            )
                        )
                    elif ".Eventhouse" in parameter["source"]:
                        pipeline_parameter["defaultValue"] = (
                            self.get_mapping_table_new_from_type_item(
                                "Kusto Ingest Uri",
                                (
                                    parameter["source"]
                                    if self.eventhouse_name == ""
                                    or self.eventhouse_name is None
                                    else f"{self.eventhouse_name}.Eventhouse"
                                ),
                            )
                        )
                elif parameter["type"] == "kusto_database":
                    pipeline_parameter = properties.get(parameter["name"], {})
                    pipeline_parameter["defaultValue"] = str(
                        parameter["source"]
                    ).replace(".KQLDatabase", "")
                elif parameter["type"] == "variable":
                    pipeline_parameter = properties.get(parameter["name"], {})
                    pipeline_parameter["defaultValue"] = self.pipeline_parameters[
                        parameter["source"]
                    ]

        with open(property_file, "w", encoding="utf-8") as file:
            json.dump(content, file, indent=4)

    def add_variable_library_default(self, folder_path, it_parameters):
        property_file = f"{folder_path}/variables.json"

        new_variables = []

        for it_parameter in it_parameters:
            if it_parameter["type"] == "kusto_query_uri":
                if ".KQLDatabase" in it_parameter["source"]:
                    new_variable = self.get_mapping_table_parent_type(
                        "KQL DB Eventhouse", it_parameter["source"], "Kusto Query Uri"
                    )
                elif ".Eventhouse" in it_parameter["source"]:
                    eventhouse_name = (
                        it_parameter["source"]
                        if self.eventhouse_name == "" or self.eventhouse_name is None
                        else f"{self.eventhouse_name}.Eventhouse"
                    )
                    new_variable = self.get_mapping_table_new_from_type_item(
                        "Kusto Query Uri", eventhouse_name
                    )
            elif it_parameter["type"] == "kusto_ingest_uri":
                if ".KQLDatabase" in it_parameter["source"]:
                    new_variable = self.get_mapping_table_parent_type(
                        "KQL DB Eventhouse", it_parameter["source"], "Kusto Ingest Uri"
                    )
                elif ".Eventhouse" in it_parameter["source"]:
                    eventhouse_name = (
                        it_parameter["source"]
                        if self.eventhouse_name == "" or self.eventhouse_name is None
                        else f"{self.eventhouse_name}.Eventhouse"
                    )
                    new_variable = self.get_mapping_table_new_from_type_item(
                        "Kusto Ingest Uri", eventhouse_name
                    )
            elif it_parameter["type"] == "kusto_database":
                new_variable = str(it_parameter["source"]).replace(".KQLDatabase", "")
            elif it_parameter["type"] == "variable":
                new_variable = self.pipeline_parameters[it_parameter["source"]]
            new_variables.append(
                {
                    "name": it_parameter.get("name"),
                    "note": it_parameter.get("name"),
                    "type": "String",
                    "value": new_variable,
                }
            )

        with open(property_file, "r", encoding="utf-8") as file:
            content = json.load(file)

        content["variables"] = new_variables

        with open(property_file, "w", encoding="utf-8") as file:
            json.dump(content, file, indent=4)

    def replace_pipeline_activities(self, folder_path, it_acitivities):
        property_file = f"{folder_path}/pipeline-content.json"
        with open(property_file, "r", encoding="utf-8") as file:
            content = json.load(file)
            activities = content.get("properties", {}).get("activities", [])
            for activity in activities:
                if activity["type"] == "TridentNotebook":
                    filtered_data = list(
                        filter(
                            lambda act: act["name"] == activity.get("name"),
                            it_acitivities,
                        )
                    )
                    if len(filtered_data) > 0:
                        activity["typeProperties"]["workspaceId"] = (
                            self.get_mapping_table_new_from_type_item(
                                "Workspace Id", self.trg_workspace_name
                            )
                        )
                        activity["typeProperties"]["notebookId"] = (
                            self.get_mapping_table_new_from_type_item(
                                "Notebook ID", filtered_data[0].get("itemName")
                            )
                        )
        with open(property_file, "w", encoding="utf-8") as file:
            json.dump(content, file, indent=4)

    def fab_get_eventstream_connection_string(self, name, connection_name):
        connection_id = ""
        item_id = self.fab_get_id(name)

        item = self.run_fab_command(
            f"api -X get /workspaces/{self.trg_workspace_id}/eventstreams/{item_id}/topology",
            capture_output=True,
            silently_continue=True,
        )
        topology = json.loads(item)

        sources = topology.get("text", {}).get("sources", [])
        source_id = list(
            filter(lambda source: source["name"] == connection_name, sources)
        )
        if len(source_id):
            connection_id = source_id[0].get("id")

        destinations = topology.get("text", {}).get("destinations", [])
        destination_id = list(
            filter(
                lambda destination: destination["name"] == connection_name, destinations
            )
        )
        if len(destination_id):
            connection_id = destination_id[0].get("id")

        connection = self.run_fab_command(
            f"api -X get /workspaces/{self.trg_workspace_id}/eventstreams/{item_id}/sources/{connection_id}/connection",
            capture_output=True,
            silently_continue=True,
        )
        connection = json.loads(connection)
        connection = (
            connection.get("text", {})
            .get("accessKeys", {})
            .get("primaryConnectionString")
        )
        return connection



from __future__ import annotations

from pathlib import Path

from typer.testing import CliRunner

from fabric_deployment_tool import _fab_cli


runner = CliRunner()


def test_cli_version() -> None:
    result = runner.invoke(_fab_cli.app, ["--version"])
    assert result.exit_code == 0
    assert result.stdout.strip()


def test_cli_run(tmp_path: Path) -> None:
    (tmp_path / "proj").mkdir()
    (tmp_path / "proj" / "artifact.yaml").write_text("{}", encoding="utf-8")
    result = runner.invoke(_fab_cli.app, ["run", str(tmp_path / "proj")])
    assert result.exit_code == 0
    assert "artifact.yaml" in result.stdout



import requests
import zipfile
import os
from io import BytesIO
import re


class fdtGit:
    def set_github(
        self, repo_owner="", repo_name="", branch="", folder_prefix="", github_token=""
    ):
        self.repo_owner = repo_owner
        self.repo_name = repo_name
        self.branch = branch
        self.folder_prefix = folder_prefix
        self.github_token = github_token

    def download_folder_as_zip(
        self,
        repo_owner,
        repo_name,
        output_zip,
        branch="main",
        folder_to_extract="src",
        folder_prefix="",
        github_token="",
    ):
        # Construct the URL for the GitHub API to download the repository as a zip file
        url = f"https://api.github.com/repos/{repo_owner}/{repo_name}/zipball/{branch}"
        headers = None

        if github_token is not None and github_token != "":
            # Replace with your actual GitHub token
            headers = {
                "Authorization": f"token {github_token}",
                "Accept": "application/vnd.github.v3+json",
            }

        response = requests.get(url, headers=headers)
        response.raise_for_status()

        folder_to_extract = (
            f"/{folder_to_extract}"
            if folder_to_extract[0] != "/"
            else folder_to_extract
        )

        # Ensure the directory for the output zip file exists
        os.makedirs(os.path.dirname(output_zip), exist_ok=True)

        # Create a zip file in memory
        with zipfile.ZipFile(BytesIO(response.content)) as zipf:
            with zipfile.ZipFile(output_zip, "w") as output_zipf:
                for file_info in zipf.infolist():
                    parts = file_info.filename.split("/")
                    if re.sub(r"^.*?/", "/", file_info.filename).startswith(
                        folder_to_extract
                    ):
                        # Extract only the specified folder
                        file_data = zipf.read(file_info.filename)
                        if folder_prefix != "":
                            for remove_folder_prefix_folder in folder_prefix.split("/"):
                                parts.remove(remove_folder_prefix_folder)
                        output_zipf.writestr(("/".join(parts[1:])), file_data)

    def uncompress_zip_to_folder(self, zip_path, extract_to):
        # Ensure the directory for extraction exists
        os.makedirs(extract_to, exist_ok=True)

        # Uncompress all files from the zip into the specified folder
        with zipfile.ZipFile(zip_path, "r") as zip_ref:
            zip_ref.extractall(extract_to)

        # Delete the original zip file
        os.remove(zip_path)



from __future__ import annotations

from pathlib import Path

import pytest

from fabric_deployment_tool import _git


def test_discover_artifacts(tmp_path: Path) -> None:
    (tmp_path / "proj").mkdir()
    files = ["a.yaml", "b.yml", "c.json", "notes.txt"]
    for name in files:
        target = tmp_path / "proj" / name
        target.write_text("{}", encoding="utf-8")
    artifacts = _git.discover_artifacts(tmp_path / "proj")
    assert artifacts == ["a.yaml", "b.yml", "c.json"]


def test_deploy_missing_root(tmp_path: Path) -> None:
    with pytest.raises(FileNotFoundError):
        _git.deploy(tmp_path / "missing")


def test_deploy_with_artifacts(tmp_path: Path) -> None:
    (tmp_path / "proj").mkdir()
    result = _git.deploy(tmp_path / "proj", artifacts=["abc.yaml"])
    assert result.deployed_items == ["abc.yaml"]
    assert "abc.yaml" in result.summary()



"""Fabric deployment tool template package."""

from __future__ import annotations

from importlib.metadata import version

import os
import json
from ._fab_cli import fdtCLU
from ._git import fdtGit
from ._util import fdtUtils
from ._fab_item_management import fdtItemManagement
import notebookutils

__all__ = [
    "__version__",
    "FabDeploymentTool",
]

try:
    __version__ = version("fabric-deployment-tool")
except Exception:  # pragma: no cover
    __version__ = "0.0.0"


class FabDeploymentTool(fdtCLU, fdtGit, fdtUtils, fdtItemManagement):
    src_workspace_id = ""
    src_workspace_name = ""
    trg_workspace_id = ""
    trg_workspace_name = ""
    deployment_order = []
    mapping_table = []
    workspace_name = ""
    capacity_name = ""
    eventhouse_name = ""
    repo_owner = ""
    repo_name = ""
    branch = ""
    folder_prefix = ""
    github_token = ""
    pipeline_parameters = {}

    def __init__(self):
        # Set environment parameters for Fabric CLI
        token = notebookutils.credentials.getToken("pbi")
        os.environ["FAB_TOKEN"] = token
        os.environ["FAB_TOKEN_ONELAKE"] = token

    def run(
        self,
        workspace_name,
        capacity_name="",
        eventhouse_name="",
        exclude=[],
        type_exclude=[],
        pipeline_parameters={},
    ):
        self.download_folder_as_zip(
            self.repo_owner,
            self.repo_name,
            output_zip="./builtin/src/src.zip",
            branch=self.branch,
            folder_to_extract=f"{self.folder_prefix}/src",
            folder_prefix=f"{self.folder_prefix}",
            github_token=self.github_token,
        )
        self.download_folder_as_zip(
            self.repo_owner,
            self.repo_name,
            output_zip="./builtin/config/config.zip",
            branch=self.branch,
            folder_to_extract=f"{self.folder_prefix}/config",
            folder_prefix=f"{self.folder_prefix}",
            github_token=self.github_token,
        )
        self.uncompress_zip_to_folder(
            zip_path="./builtin/config/config.zip", extract_to="./builtin"
        )
        base_path = "./builtin/"

        self.eventhouse_name = eventhouse_name
        self.pipeline_parameters = pipeline_parameters
        deploy_order_path = os.path.join(base_path, "config/deployment_order.json")
        with open(deploy_order_path, "r") as file:
            self.deployment_order = json.load(file)

        # deploy workspace idempotent
        if "NotFound" in self.fab_workspace_exists(f"{workspace_name}.Workspace"):
            if capacity_name == "" or capacity_name is None:
                raise "Workspace doesn´t exist and capacity_name not provided"
            self.run_fab_command(
                f"mkdir {workspace_name}.Workspace -P capacityname={capacity_name}.Capacity"
            )
            print(f"New Workspace Create", flush=True)

        self.src_workspace_name = "Workspace.src"
        self.src_workspace_id = self.get_id_by_name(self.src_workspace_name)
        self.trg_workspace_id = self.fab_get_workspace_id(f"{workspace_name}.Workspace")
        self.trg_workspace_name = f"{workspace_name}.Workspace"

        print(f"Target Workspace Id: {self.trg_workspace_id}", flush=True)
        print(f"Target Workspace Name: {self.trg_workspace_name}", flush=True)

        self.mapping_table.append(
            {
                "Type": "Workspace Id",
                "Item": self.trg_workspace_name,
                "old": self.get_id_by_name(self.src_workspace_name),
                "new": self.trg_workspace_id,
            }
        )
        self.mapping_table.append(
            {
                "Type": "Workspace Blank Id",
                "Item": self.trg_workspace_name,
                "old": "00000000-0000-0000-0000-000000000000",
                "new": self.trg_workspace_id,
            }
        )
        self.mapping_table.append(
            {
                "Type": "Workspace Name",
                "Item": self.trg_workspace_name,
                "old": self.src_workspace_name,
                "new": self.trg_workspace_name.replace(".Workspace", ""),
            }
        )
        exclude = exclude + [self.src_workspace_name]

        for it in self.deployment_order:
            new_id = None
            name = it["name"]
            type = it.get("type")

            if name in exclude:
                continue

            if type in type_exclude:
                continue

            self.__deploy_item(name, None, it)

            for child in it.get("children", []):
                child_name = child["name"]
                self.__deploy_item(name, child_name, child)

    def __deploy_item(self, name, child=None, it=None):
        parent = ""
        cli_parameter = ""

        # Copy and replace IDs in the item
        tmp_path = self.copy_to_tmp(name, child, it.get("type"))

        if child is not None:
            parent = name
            name = child

        if ".KQLDatabase" in name:
            if child is not None:
                parent = (
                    parent
                    if self.eventhouse_name == "" or self.eventhouse_name is None
                    else f"{self.eventhouse_name}.Eventhouse"
                )
            if it["parent"] is not None:
                parent = (
                    it["parent"]
                    if self.eventhouse_name == "" or self.eventhouse_name is None
                    else f"{self.eventhouse_name}.Eventhouse"
                )
            self.mapping_table.append(
                {
                    "Type": "KQL DB Eventhouse",
                    "Item": name,
                    "old": it["parent"],
                    "new": parent,
                }
            )
            self.replace_kqldb_parent_eventhouse(tmp_path, parent)
        elif ".Eventhouse" in name:
            name = (
                name
                if self.eventhouse_name == "" or self.eventhouse_name is None
                else f"{self.eventhouse_name}.Eventhouse"
            )
        elif ".Eventstream" in name:
            self.replace_eventstream_destination(tmp_path, it.get("destinations"))
        elif ".Notebook" in name:
            cli_parameter = cli_parameter + " --format .py"
        elif ".DataPipeline" in name:
            self.replace_pipeline_parameter(tmp_path, it.get("parameters", []))
            self.replace_pipeline_activities(tmp_path, it.get("activities", []))
        elif ".SemanticModel" in name:
            self.replace_ids_in_folder(tmp_path, self.mapping_table)
        elif ".KQLDashboard" in name:
            self.replace_kqldashboard_datasources(tmp_path, it.get("datasources"))
        elif ".KQLQueryset" in name:
            self.replace_kqlqueryset_datasources(tmp_path, it.get("datasources"))
        elif ".Report" in name:
            self.replace_pbi_report_definition(tmp_path, it.get("datasource"))
        elif ".VariableLibrary" in name:
            self.add_variable_library_default(tmp_path, it.get("variables"))

        self.run_fab_command(
            f"import  /{self.trg_workspace_name}/{name} -i {tmp_path} -f {cli_parameter} ",
            silently_continue=True,
        )

        new_id = self.fab_get_id(name)

        if ".KQLDatabase" in name:
            self.mapping_table.append(
                {"Type": "KQL DB ID", "Item": name, "old": it["id"], "new": new_id}
            )
        elif ".Eventhouse" in name:
            query_uri = self.fab_get_kusto_query_uri(name)
            ingest_uri = self.fab_get_kusto_ingest_uri(name)
            self.mapping_table.append(
                {
                    "Type": "Kusto Query Uri",
                    "Item": name,
                    "old": it["kustoQueryUri"],
                    "new": query_uri,
                }
            )
            self.mapping_table.append(
                {
                    "Type": "Kusto Ingest Uri",
                    "Item": name,
                    "old": it["kustoIngestUri"],
                    "new": ingest_uri,
                }
            )
            self.mapping_table.append(
                {"Type": "Eventhouse ID", "Item": name, "old": it["id"], "new": new_id}
            )
        elif ".Eventstream" in name:
            for customEndpointName in it.get("customEndpointName", []):
                self.mapping_table.append(
                    {
                        "Type": "Connection String Eventstream",
                        "Item": name,
                        "old": customEndpointName,
                        "new": self.fab_get_eventstream_connection_string(
                            name, customEndpointName
                        ),
                    }
                )
            self.mapping_table.append(
                {"Type": "Eventstream ID", "Item": name, "old": it["id"], "new": new_id}
            )
        elif ".Notebook" in name:
            self.mapping_table.append(
                {"Type": "Notebook ID", "Item": name, "old": it["id"], "new": new_id}
            )
        elif ".DataPipeline" in name:
            self.mapping_table.append(
                {"Type": "Pipeline ID", "Item": name, "old": it["id"], "new": new_id}
            )
        elif ".Report" in name:
            self.mapping_table.append(
                {"Type": "Report ID", "Item": name, "old": it["id"], "new": new_id}
            )
        elif ".SemanticModel" in name:
            self.mapping_table.append(
                {
                    "Type": "Semantic Model ID",
                    "Item": name,
                    "old": it["id"],
                    "new": new_id,
                }
            )
        elif ".KQLDashboard" in name:
            self.mapping_table.append(
                {
                    "Type": "KQLDashboard ID",
                    "Item": name,
                    "old": it["id"],
                    "new": new_id,
                }
            )

        if it.get("folder") is not None:
            self.fab_update_item_folder(name, it.get("folder"))



import shutil
import os
import sempy.fabric as fabric
import json
import uuid
from zipfile import ZipFile


class fdtUtils:
    def get_id_by_name(self, name):
        for it in self.deployment_order:
            if it.get("name") == name:
                return it.get("id")
        return None

    def get_schedule_by_name(self, name):
        for it in self.deployment_order:
            if it.get("name") == name:
                return it.get("schedule")
        return None

    def copy_to_tmp(self, name, child=None, type=None):
        child_path = "" if child is None else f".children/{child}/"
        type_path = "" if type is None else f"{type}/"
        shutil.rmtree("./builtin/tmp", ignore_errors=True)
        path2zip = "./builtin/src/src.zip"
        with ZipFile(path2zip) as archive:
            for file in archive.namelist():
                if file.startswith(f"src/{type_path}{name}/{child_path}"):
                    archive.extract(file, "./builtin/tmp")
        return f"./builtin/tmp/src/{type_path}{name}/{child_path}"

    def get_mapping_table_new_from_type(self, type):
        result = ""
        filtered_data = list(
            filter(lambda item: item["Type"] == type, self.mapping_table)
        )
        if len(filtered_data) > 0:
            result = filtered_data[0]["new"]
        return result

    def get_mapping_table_new_from_old(self, old):
        result = ""
        filtered_data = list(
            filter(lambda item: item["old"] == old, self.mapping_table)
        )
        if len(filtered_data) > 0:
            result = filtered_data[0]["new"]
        return result

    def get_mapping_table_new_from_type_item(self, type, item):
        result = ""
        filtered_data = list(
            filter(
                lambda table: table["Type"] == type and table["Item"] == item,
                self.mapping_table,
            )
        )
        if len(filtered_data) > 0:
            result = filtered_data[0]["new"]
        return result

    def get_mapping_table_parent_type(self, type, item, parent_type):
        parent_item = self.get_mapping_table_new_from_type_item(type, item)
        result = self.get_mapping_table_new_from_type_item(parent_type, parent_item)
        return result

    def replace_ids_in_folder(self, folder_path, mapping_table):
        for root, _, files in os.walk(folder_path):
            for file_name in files:
                if file_name.endswith(
                    (".py", ".json", ".pbir", ".platform", ".ipynb", ".py", ".tmdl")
                ) and not file_name.endswith("report.json"):
                    file_path = os.path.join(root, file_name)
                    with open(file_path, "r", encoding="utf-8") as file:
                        content = file.read()
                        for mapping in mapping_table:
                            content = content.replace(mapping["old"], mapping["new"])
                    with open(file_path, "w", encoding="utf-8") as file:
                        file.write(content)

    def fab_update_environments_spark_monitor(
        self, evironments, workspace_name, eventstream_name, connection_name
    ):

        for environment in evironments:
            workspace = environment.get("workspace_id")
            environment = environment.get("environment_id")
            print("################### UPDATING ENVIRONMENT")
            print("workspace: " + workspace)
            print("Environment: " + environment)

            self.trg_workspace_id = self.fab_get_workspace_id(
                f"{workspace_name}.Workspace"
            )
            self.trg_workspace_name = f"{workspace_name}.Workspace"
            eventstream_name = (
                f"{eventstream_name}.Eventstream"
                if ".Eventstream" not in eventstream_name
                else eventstream_name
            )
            connection_string = self.fab_get_eventstream_connection_string(
                eventstream_name, connection_name
            )
            StringSparkProperties = json.dumps(
                {
                    "sparkProperties": {
                        "spark.synapse.diagnostic.emitters": "SparkEmitter",
                        "spark.synapse.diagnostic.emitter.SparkEmitter.type": "AzureEventHub",
                        "spark.synapse.diagnostic.emitter.SparkEmitter.secret": connection_string,
                        "spark.fabric.pools.skipStarterPools": "true",
                    }
                }
            )
            response = self.run_fab_command(
                f"api -X patch /workspaces/{workspace}/environments/{environment}/staging/sparkcompute -i  {StringSparkProperties}",
                capture_output=True,
                silently_continue=True,
            )
            response = self.run_fab_command(
                f"api -X post /workspaces/{workspace}/environments/{environment}/staging/publish ",
                capture_output=True,
                silently_continue=True,
            )
            print("################### FINISH UPDATING ENVIRONMENT")

    def update_capcity_events_eventstream(self, workspace, item_name="CapacityEvents"):
        tmp_path = "./builtin/tmp/export/"

        shutil.rmtree(tmp_path, ignore_errors=True)

        os.makedirs(os.path.dirname(tmp_path), exist_ok=True)

        self.run_fab_command(
            f"export  /{workspace}.Workspace/{item_name}.Eventstream -o {tmp_path} -f ",
            silently_continue=True,
        )

        property_file = f"{tmp_path}{item_name}.Eventstream/eventstream.json"

        new_sources = []
        new_input_nodes = []

        dfCapacities = fabric.list_capacities()
        dfCapacities = dfCapacities.query("Sku != 'PP3'")

        with open(property_file, "r", encoding="utf-8") as file:
            content = json.load(file)
            sources = content.get("sources", [])
            for index, row in dfCapacities.iterrows():
                capacity_id = row["Id"]
                name = row["Display Name"]
                sku = row["Sku"]
                name = f"{name.replace(' ','')}-{sku}"
                name = name.replace("_", "")
                filtered_data = list(
                    filter(
                        lambda table: table.get("properties", {}).get("capacityId")
                        == capacity_id,
                        sources,
                    )
                )
                if len(filtered_data) > 0:
                    new_source = filtered_data.pop()
                    new_input_node = {"name": new_source.get("name")}
                else:
                    new_source = {
                        "id": str(uuid.uuid4()),
                        "name": name,
                        "type": "FabricCapacityUtilizationEvents",
                        "properties": {
                            "eventScope": "Capacity",
                            "capacityId": capacity_id,
                            "includedEventTypes": [
                                "Microsoft.Fabric.Capacity.State",
                                "Microsoft.Fabric.Capacity.Summary",
                            ],
                            "filters": [],
                        },
                    }
                    new_input_node = {"name": name}
                new_sources.append(new_source)
                new_input_nodes.append(new_input_node)
            content["sources"] = new_sources

            for stream in content["streams"]:
                if stream["type"] == "DefaultStream":
                    stream["inputNodes"] = new_input_nodes

        with open(property_file, "w", encoding="utf-8") as file:
            json.dump(content, file, indent=4)

        self.run_fab_command(
            f"import  /{workspace}.Workspace/{item_name}.Eventstream -i {tmp_path}/{item_name}.Eventstream -f ",
            silently_continue=True,
        )
